@@ -65,14 +65,42 @@ class SoozAccess(object):
     # ENTERPRISE
     def post_enterprise(self, name: str):
         """Create an enterprise."""
-        raise NotImplementedError("Not implemented yet")
 
-    def get_enterprise_factors(self, enterprise: int):
+        content = {"name": name}
+
+        try:
+            r = self.session.post(
+                f"{SoozAccess.host}/enterprises", json=content
+            )
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Post Enterprise", r)
+
+        return content, r
+
+    def get_enterprise_factors(self, enterprise_id: int):
         """Get all factors with selected factors of an enterprise."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/enterprises/{enterprise_id}/factors")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Enterprise Factors", r)
+
+        return content, r
 
     def put_entreprise_factors(self, enterprise: int):
-        """Purge and recreate selected factors of an enterprise."""
+        """Purge and recreate selected factors of an enterprise."""        
         raise NotImplementedError("Not implemented yet")
 
     # FACTORS
@@ -95,7 +123,20 @@ class SoozAccess(object):
     # GROUP MEMBERS
     def get_invitable_users(self, group: int):
         """Return users that can be invited to a group."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/groups/{group}/invitableUsers")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Invitable Users", r)
+
+        return content, r
 
     def post_group_invite(self, group: int, ids: list[int]) -> Tuple[dict, Response]:
         """Invite users to a specific group."""
@@ -119,6 +160,7 @@ class SoozAccess(object):
         self, group: int, status: str = "accepted"
     ) -> Tuple[dict, Response]:
         """Accept/decline an invitation to join a group."""
+
         content = {"status": status}
 
         try:
@@ -135,11 +177,36 @@ class SoozAccess(object):
 
     def get_group_members(self, id: int):
         """Return members of a group."""
-        raise NotImplementedError("Not implemented yet")
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/groups/{id}/members")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Groups Members", r)
+
+        return content, r
 
     def delete_group_member(self, group: int, user: int):
         """Remove a user from a group."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.delete(f"{SoozAccess.host}/groups/{group}/members/{user}")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Groups", r)
+
+        return content, r
 
     # GROUPS
     def get_user_groups(self) -> Tuple[dict, Response]:
@@ -180,7 +247,20 @@ class SoozAccess(object):
 
     def put_group(self, group: int, name: str, description: str):
         """Update a group (requires the user to be admin of the group)."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {"name": name,"description": description,}
+
+        try:
+            r = self.session.put(
+                f"{SoozAccess.host}/groups/{group}", json=content
+            )
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        finally:
+            self.show_status("    Put Group", r)
+
+        return {}, r
 
     def delete_group(self, group: int) -> Tuple[dict, Response]:
         """Delete a group (requires the user to be admin of the group)."""
@@ -201,7 +281,19 @@ class SoozAccess(object):
     # LEVELS
     def get_levels(self):
         """Return the current level of the user for gamification. The points are increased by 25 with each PUT to the themes-responses endpoint."""
-        raise NotImplementedError("Not implemented yet")
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/levels")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Levels", r)
+        
+        return content, r
 
     # RESPONSE OPTIONS
     def get_oneshot(self) -> Tuple[dict, Response]:
@@ -221,11 +313,39 @@ class SoozAccess(object):
 
     def get_factor_response_options(self, theme: int, factor: int):
         """Get all possible responses for a particular factor."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(
+                f"{SoozAccess.host}/themes/{theme}/factors/{factor}/responseOptions"
+            )
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Factor Response Options", r)
+
+        return content, r
 
     def get_theme_response_options(self, theme: int):
         """Get the responseOptions to the quetsion of a theme."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/themes/{theme}/responseOptions")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Theme Response Options", r)
+
+        return content, r
 
     # RESPONSES
     def get_responses(self) -> Tuple[dict, Response]:
@@ -243,9 +363,41 @@ class SoozAccess(object):
 
         return content, r
 
+    def get_all_responses(self, since: str, to:str) -> Tuple[dict, Response]:
+        """Get all responses of the user for a specific period of time."""
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/oneshot/responses/since/{since}/to/{to}")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get All Responses", r)
+
+        return content, r
+
     def get_factor_responses(self, theme: int, factor: int):
         """Get the responses of the user for today to this factor and for the given period."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(
+                f"{SoozAccess.host}/themes/{theme}/factors/{factor}/responses"
+            )
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Factor Responses", r)
+            
+        return content, r
 
     def put_response(
         self, theme: int, factor: int, value: int, date: str
@@ -274,15 +426,54 @@ class SoozAccess(object):
 
     def delete_response(self, theme: int, factor: int, response: int):
         """Delete the given response."""
-        raise NotImplementedError("Not implemented yet")
+
+        try:
+            r = self.session.delete(
+                f"{SoozAccess.host}/themes/{theme}/factors/{factor}/responses/{response}"
+            )
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        finally:
+            self.show_status("    Delete Response", r)
+
+        return {}, r
+
 
     def get_all_themes_responses(self):
         """Get responses of today to all themes for the user."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/themes/responses")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get All Themes Responses", r)
+
+        return content, r
+
 
     def get_theme_responses(self, theme: int):
         """Get todays response of the user for given theme (only used for the environment category question)."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/themes/{theme}/responses")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get All Themes Responses", r)
+
+        return content, r
 
     def put_theme_response(self, theme: int):
         """Update the crrent response of the user. Key on the columns (userId, themeId, period, date)."""
@@ -302,6 +493,8 @@ class SoozAccess(object):
         group: int or list[int] = None,
     ):
         """Returns the number of responses per responseId for one factor."""
+
+
         raise NotImplementedError("Not implemented yet")
 
     def get_stats_days(
@@ -370,13 +563,26 @@ class SoozAccess(object):
         else:
             content = json.loads(r.content.decode("utf-8"))
         finally:
-            self.show_status("    Themes", r)
+            self.show_status("    Get Themes", r)
 
         return content, r
 
     def get_theme(self, theme: int):
         """Get a specific theme."""
-        raise NotImplementedError("Not implemented yet")
+
+        content = {}
+
+        try:
+            r = self.session.get(f"{SoozAccess.host}/themes/{theme}")
+            r.raise_for_status()
+        except HTTPError as e:
+            pass
+        else:
+            content = json.loads(r.content.decode("utf-8"))
+        finally:
+            self.show_status("    Get Theme", r)
+
+        return content, r
 
     # USERS
     def post_user(self, entreprise_id: int = 1) -> Tuple[dict, Response]:
